#include <cstdlib>
#include <cmath>
#include "3DWindow.h"

enum menuenties { ACTION,TRANSLATE,ZOOM_IN,ZOOM_OUT,QUIT };

void ThreeDWindow::show()

{ int    x,y;
  int    wx,wy;
  int    rx0,ry0;
  float  value;
  const int wmax = 400;
  static float colors[3*2*wmax];
  static float normals[3*2*wmax];
  static float vertices[3*2*wmax];
  static int   indices[2*wmax];
  int    i;
  
  rx0 = int(floor((llcorner.x-RD.xllcorner)/RD.cellsize));
  ry0 = int(floor((llcorner.y-RD.yllcorner)/RD.cellsize));

  wx = int(ceil(width/scale/RD.cellsize));
  if (wx > wmax) wx = wmax;
  wy = int(ceil(height/scale/RD.cellsize));
  if (wy > wmax) wy = wmax;

  glEnableClientState(GL_COLOR_ARRAY);
  glEnableClientState(GL_NORMAL_ARRAY);
  glEnableClientState(GL_VERTEX_ARRAY);

  glColorPointer(3,GL_FLOAT,0,colors);
  glNormalPointer(GL_FLOAT,0,normals);
  glVertexPointer(3,GL_FLOAT,0,vertices);

  for(y = 0; y < wy; ++y)
  {
    i = (y % 2)? 0 : 3*wx;
    for(x = 0; x < wx; ++x,i += 3)
    {
      if (rx0+x >= 0 && rx0+x < RD.ncols && ry0+y >= 0 && ry0+y < RD.nrows-1) 
      {
        value = RD(rx0+x,ry0+y); 

//      normals[i+0] = normals[i+1] = 0.0; normals[i+2] = 1.0;
        RD.normal(rx0+x,ry0+y,normals+i);

//      colors[i+0] = colors[i+1] = colors[i+2] = 1.0;

        colors[i+0] = 0.46875+0.625*value/1024.0;
        colors[i+1] = 1.0-3.375*value/1024.0;
        colors[i+2] = 0.0+0.625*value/1024.0;

      }
      else
      { 
        value = 0;
        normals[i] = normals[i+1] = 0.0;
        normals[i+2] = 1.0;
        colors[i] = colors[i+1] = colors[i+2] = 0.0;
      }

      vertices[i+0] = RD.xllcorner+RD.cellsize*(rx0+x);
      vertices[i+1] = RD.yllcorner+RD.cellsize*(ry0+y);
      vertices[i+2] = 0.0; // value;
    }

    if (y == 0) continue;

    for(i=0,x=0;x<wx;++x)
    { indices[i++] = x+(y%2)*wx;
      indices[i++] = x+(1-y%2)*wx;
    }

    glDrawElements(GL_TRIANGLE_STRIP,2*wx,GL_UNSIGNED_INT,indices);
    glEnd();
  }
}

ThreeDWindow::ThreeDWindow(GlutAdmin &admin,int initW,int initH,int initX,int initY,char* title)
{
   width = initW;
   height = initH;
   op_mode = TRANSLATE;
   scaling = false;
 
   glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE ); // | GLUT_DEPTH);
   glutInitWindowSize(width,height);
   glutInitWindowPosition(initX,initY);

   admin.CreateWindow(title,this);

   int submenu = glutCreateMenu(admin.MenuFunc);
   glutAddMenuEntry("Translate",TRANSLATE);
   glutAddMenuEntry("Zoom in",ZOOM_IN);
   glutAddMenuEntry("Zoom out",ZOOM_OUT);
   int mainmenu = glutCreateMenu(admin.MenuFunc);
   glutAddSubMenu("Action",submenu);
   glutAddMenuEntry("Quit",QUIT);

   glutAttachMenu(GLUT_RIGHT_BUTTON);

   mx0 = my0 = 0;
}

ThreeDWindow::~ThreeDWindow()
{
   glutDestroyWindow(ID);
}

void ThreeDWindow::MenuFunc(int value)
{
  if (value == QUIT) exit(EXIT_SUCCESS);
  op_mode = value;
}

void ThreeDWindow::MouseFunc(int button, int state, int x, int y)
{
  if (button != GLUT_LEFT_BUTTON) return;

  if (state == GLUT_DOWN)
  { 
    mx0 = x; 
    my0 = height-y;
  }

  if (state == GLUT_UP)
  { 
    if (op_mode == TRANSLATE)
    {
      llcorner.x += (mx0-x)/scale;
      llcorner.y += (my0-height+y)/scale;

      mx0 = x;
      my0 = height-y;

      glutPostRedisplay();
    }
  }

  if (op_mode == ZOOM_IN || op_mode == ZOOM_OUT)
    scaling = (state == GLUT_DOWN);
}

/*
void ThreeDWindow::MotionFunc(int x, int y)
{
}
*/

void ThreeDWindow::IdleFunc()
{
  if (scaling)
  {
    if (op_mode == ZOOM_IN)
    {
      scale *= 1.05;
      llcorner.x += mx0*0.05/scale;
      llcorner.y += my0*0.05/scale;
    }
    if (op_mode == ZOOM_OUT)
    {
      llcorner.x -= mx0*0.05/scale;
      llcorner.y -= my0*0.05/scale;
      scale /= 1.05;
    }

    glutPostRedisplay();
  }
}

void ThreeDWindow::DisplayFunc()

{ 
  static GLfloat pos[4] = { 0.0, -0.5, 1.0, 0.0 };
  static GLfloat black[4] = {0.0, 0.0, 0.0, 1.0 };

  glClear( GL_COLOR_BUFFER_BIT ); // | GL_DEPTH_BUFFER_BIT );

  glLightfv( GL_LIGHT0, GL_POSITION, pos );
//  glEnable( GL_CULL_FACE );
//  glEnable( GL_DEPTH_TEST );

  glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
//  glShadeModel(GL_FLAT);
  glShadeModel(GL_SMOOTH);

  glEnable( GL_LIGHTING );
  glEnable( GL_LIGHT0 );
  glEnable( GL_COLOR_MATERIAL );
  glEnable( GL_NORMALIZE );

  glPushMatrix();
  glScalef(scale,scale,scale);
  glTranslatef(-llcorner.x,-llcorner.y,0.0);
/*
  glDisable( GL_LIGHTING );
  glDisable( GL_LIGHT0 );
  glPolygonMode(GL_FRONT,GL_LINE);
*/
  show();

  glPopMatrix();

  glutSwapBuffers();
}

void ThreeDWindow::ReshapeFunc(int newwidth, int newheight)
{  
  width = newwidth; 
  height = newheight;
  glViewport(0, 0, GLint(width), GLint(height));
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0,GLdouble(width),0,GLdouble(height),-100.0,100.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

