#include <cstdlib>
#include <cmath>
//#include <minmax.h>
#include "TinWindow.h"

enum menuenties { WIREFRAME,FLAT,QUIT };

void TinWindow::show()

{
  list<Triangle>::iterator i;

  glBegin( GL_TRIANGLES );
  for(i = tin.begin(); i != tin.end(); ++i)
  {
    Point3D &a = i->a;
    Point3D &b = i->b;
    Point3D &c = i->c;

    Point3D n;

    n.x = a.y*b.z - a.z*b.y + b.y*c.z - b.z*c.y + c.y*a.z - c.z*a.y;
    n.y = a.z*b.x - a.x*b.z + b.z*c.x - b.x*c.z + c.z*a.x - c.x*a.z;
    n.z = a.x*b.y - a.y*b.x + b.x*c.y - b.y*c.x + c.x*a.y - c.y*a.x;

    glNormal3f(n.x,n.y,n.z);

//  glColor3f(0.46875+0.625*a.z/4096.0,1.0-3.375*a.z/4096.0,0.625*a.z/4096.0);
    glVertex3f(a.x,a.y,a.z);
//  glColor3f(0.46875+0.625*b.z/4096.0,1.0-3.375*b.z/4096.0,0.625*b.z/4096.0);
    glVertex3f(b.x,b.y,b.z);
//  glColor3f(0.46875+0.625*c.z/4096.0,1.0-3.375*c.z/4096.0,0.625*c.z/4096.0);
    glVertex3f(c.x,c.y,c.z);
  }
  glEnd();
}

TinWindow::TinWindow(GlutAdmin &admin,int initW,int initH,int initX,int initY,char* title)
{
   width = initW;
   height = initH;
   angle = 0.0;
   vis_mode = WIREFRAME;
 
   glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
   glutInitWindowSize(width,height);
   glutInitWindowPosition(initX,initY);

   admin.CreateWindow(title,this);

   int mainmenu = glutCreateMenu(admin.MenuFunc);
   glutAddMenuEntry("Wireframe",WIREFRAME);
   glutAddMenuEntry("Flat Shaded",FLAT);
   glutAddMenuEntry("Quit",QUIT);

   glutAttachMenu(GLUT_RIGHT_BUTTON);

   mx0 = my0 = 0;
}

TinWindow::~TinWindow()
{
   glutDestroyWindow(ID);
}

void TinWindow::MenuFunc(int value)
{
  if (value == QUIT) exit(EXIT_SUCCESS);
  vis_mode = value;
  glutPostRedisplay();
}

void TinWindow::MouseFunc(int button, int state, int x, int y)
{
  if (button != GLUT_LEFT_BUTTON) return;
  
  if (state == GLUT_DOWN)
  {
    mx0 = x;
    my0 = height-y;
  }
/*
  if (state == GLUT_UP)
  {
    angle += my0-height+y;
    if (angle > 0.0) angle = 0.0;
    if (angle < -90.0) angle = -90.0;
    glutPostRedisplay();
  }
*/
}

void TinWindow::MotionFunc(int x,int y)
{
  angle += my0-height+y;
  mx0 = x;
  my0 = height-y;
  if (angle > 0.0) angle = 0.0;
  if (angle < -90.0) angle = -90.0;
  glutPostRedisplay();
}

void TinWindow::DisplayFunc()

{ 
  static GLfloat pos[4] = { 0.0, 0.0, 1.0, 0.0 };

  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

  glLightfv( GL_LIGHT0, GL_POSITION, pos );
//glEnable( GL_CULL_FACE );
  glEnable( GL_DEPTH_TEST );

  glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
  glShadeModel(GL_FLAT);
//glShadeModel(GL_SMOOTH);

//glEnable( GL_COLOR_MATERIAL );
  glEnable( GL_NORMALIZE );

  glPushMatrix();

  if (vis_mode == WIREFRAME)
  {
    glDisable( GL_LIGHTING );
    glDisable( GL_LIGHT0 );
    glPolygonMode(GL_FRONT,GL_LINE);
  }

  if (vis_mode == FLAT)
  {
    glEnable( GL_LIGHTING );
    glEnable( GL_LIGHT0 );
    glPolygonMode(GL_FRONT,GL_FILL);
  }

  glTranslatef(0.5*width,0.5*height,0.0);
  glRotatef(angle,1.0,0.0,0.0);
  glTranslatef(-0.5*width,-0.5*height,0.0);

  float s = min(width/(bb.max.x-bb.min.x),height/(bb.max.y-bb.min.y));

  glScalef(s,s,10.0*s);
  glTranslatef(-bb.min.x,-bb.min.y,0.0);
  show();

  glPopMatrix();

  glutSwapBuffers();
}

void TinWindow::ReshapeFunc(int newwidth, int newheight)
{  
  width = newwidth; 
  height = newheight;
  glViewport(0, 0, GLint(width), GLint(height));
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0,GLdouble(width),0,GLdouble(height),-1000.0,1000.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

